package org.xbib.interlibrary.client.gvi.sru;

import java.io.IOException;
import java.io.InputStream;
import java.io.UncheckedIOException;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Locale;
import java.util.function.Consumer;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.action.resolve.ResolveRequest;
import org.xbib.interlibrary.action.resolve.ResolveResponse;
import org.xbib.interlibrary.action.resolve.ResolveResponseBuilder;
import org.xbib.interlibrary.action.resolve.ResolveServiceResponseBuilder;
import org.xbib.interlibrary.api.action.Response;
import org.xbib.interlibrary.api.action.Service;
import org.xbib.interlibrary.api.action.ServiceArguments;
import org.xbib.interlibrary.common.DefaultBibliographicDescription;
import org.xbib.interlibrary.api.util.Lists;
import org.xbib.interlibrary.sru.AbstractSRUResolveService;
import org.xbib.interlibrary.sru.SRUClientHelper;
import org.xbib.marc.Marc;
import org.xbib.marc.MarcRecord;
import org.xbib.marc.MarcRecordListener;
import org.xbib.marc.MarcXchangeConstants;
import org.xbib.marc.xml.MarcContentHandler;

public class SRUResolveService extends AbstractSRUResolveService {

    public SRUResolveService(ServiceArguments arguments, SRUClientHelper sruClientHelper) {
        super(arguments, sruClientHelper);
    }

    @Override
    protected String supports(String source) {
        return source != null && source.toLowerCase(Locale.ROOT).contains("gvi") ? "GVI" : null;
    }

    @Override
    protected void resolve(ResolveRequest resolveRequest,
                           ResolveServiceResponseBuilder serviceResponseBuilder,
                           InputStream inputStream,
                           DefaultBibliographicDescription.Builder descriptionBuilder) {
        MarcContentHandler marcContentHandler = new MarcContentHandler();
        marcContentHandler.setMarcRecordListener(new MarcRecordListener() {
            @Override
            public void beginCollection() {

            }
            @Override
            public void record(MarcRecord marcRecord) {
                descriptionBuilder.setSource("GVI");
                descriptionBuilder.setSourceId(marcHelper.identifier(marcRecord));
                addIdentifier(descriptionBuilder, "ISBN", marcHelper.extractIBN(marcRecord, "020", null, List.of("a")));
                addIdentifier(descriptionBuilder, "ISSN", marcHelper.extractIBN(marcRecord, "022", null, List.of("a")));
                descriptionBuilder.setInTitle(Lists.firstElementAsStringIfAny(marcHelper.extractTitle(marcRecord, List.of("245"))));
                descriptionBuilder.setPublisher(String.join(", ", marcHelper.extractPublishedBy(marcRecord, List.of("260", "264"), "b")));
                if (resolveRequest.isSourceEnabled()) {
                    serviceResponseBuilder.setSource(marcRecord);
                }
                serviceResponseBuilder.setStatus(getName(), Response.Status.OK);
            }
            @Override
            public void endCollection() {

            }
        });
        try {
            Marc.builder()
                    .setInputStream(inputStream)
                    .setCharset(StandardCharsets.UTF_8)
                    .setContentHandler(new MarcContentHandler()
                            .setFormat("MarcXML")
                            .setType(MarcXchangeConstants.BIBLIOGRAPHIC_TYPE)
                            .addNamespace("http://www.loc.gov/MARC21/slim")
                            .setMarcListener(MarcXchangeConstants.BIBLIOGRAPHIC_TYPE, marcContentHandler)
                            .setMarcListener(MarcXchangeConstants.HOLDINGS_TYPE, marcContentHandler))
                    .build()
                    .xmlReader()
                    .parse();
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }
    }

    @Override
    public int compareTo(Service<ResolveRequest, ResolveResponse, ResolveResponseBuilder> o) {
        return getName().compareTo(o.getName());
    }

    protected String normalizeSourceId(String sourceId) {
        return sourceId != null ? sourceId.trim() : null;
    }

    @Override
    protected String sourceIdCQL(String sourceId, String formattedSourceId) {
        return "rec.id = \"" + sourceId + "\"";
    }

    @Override
    protected Consumer<TinyMap.Builder<String, Object>> parameterCompletion() {
        return params -> {
            params.put("maximumRecords", 1);
            params.put("stylesheet", "");
        };
    }
}
